package edu.uoc.grp.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import javax.servlet.http.HttpSession;
import java.util.Map;
import edu.uoc.grp.service.ServeiAcces;
import edu.uoc.grp.service.ServeiUsuari;
import edu.uoc.grp.exceptions.ExcepcioUsuari;
import edu.uoc.grp.model.Usuari;
import edu.uoc.grp.utilities.Utils;


/**
 * Classe ControladorAcces.
 * 
 * Classe de tipus controlador encarregada de gestionar les peticions i les respostes relacionades amb l'accs a l'aplicaci. 
 *
 * @version 1.0; java version "1.8.0";
 */
@Controller
@RequestMapping("/acces")
public class ControladorAcces {

	private static final Logger LOGGER = LogManager.getLogger(ControladorAcces.class.getName());
	
    @Autowired
    private ServeiAcces serveiAcces;	
    
    @Autowired
    private ServeiUsuari serveiUsuari;
    
    @Autowired
    private Utils utils;


	@GetMapping("/ini")
    public String accesIni(HttpSession session, Model model) {
		
		LOGGER.info("accesIni");

		//Anullaci del objecte de la sessi de l'usuari de l'aplicaci G.R.P. 2018
		if(session != null) {
			if(session.getAttribute("usuari") != null) session.removeAttribute("usuari");
			if(session.getAttribute("tipusUsuari") != null) session.removeAttribute("tipusUsuari");
		}
		
		return "login";
    }	


	@PostMapping("/login")
    public String accesLogin(HttpSession session, Model model, @ModelAttribute("usuari") Usuari usuari) {

		LOGGER.info("accesLogin");	
		LOGGER.debug("Login es: " + usuari.getLogin());

		
        Map<String, String> resultat = serveiAcces.cercarUsuariAcces(usuari.getLogin(), usuari.getContrasenya());

        //En cas d'error, mostrem a l'usuari el missatge informatiu de l'error.
        if (resultat.get("RESULTAT").equalsIgnoreCase("NOK")) {
        	
            String missatgeError = resultat.get("MISSATGE");
            LOGGER.error(missatgeError);

    		model.addAttribute("missatgeError", missatgeError);            
           
            return "error";
        }		
		
        //En cas que les dades d'accs de l'usuari siguin correctes es fica l'usuari a la sessi,
        //i a continuaci se li mostrar el men corresponen segons el seu tipus d'usuari.
        else if (resultat.get("RESULTAT").equalsIgnoreCase("OK")) {

            String tipusUsuari = resultat.get("TIPUS_USUARI");
            LOGGER.debug("TipusUsuari: " + tipusUsuari);

            //Posem el usuari dins de la sessi.
            session.setAttribute("usuari", usuari.getLogin());
            session.setAttribute("tipusUsuari", tipusUsuari);

            if ("ADMINISTRADOR".equalsIgnoreCase(tipusUsuari)) {

                return "menuAdministrador";

            } else if ("PLANIFICADOR".equalsIgnoreCase(tipusUsuari)) {

                return "menuPlanificador";

            } else if ("TRIPULANT".equalsIgnoreCase(tipusUsuari)) {
   
                return "menuTripulant";
            }
      }        
		return "login";
    }


	@GetMapping("/logout")
    public String accesLogout(HttpSession session, Model model) {
		
		LOGGER.info("accesLogout");
		
		//Anullaci del objecte de la sessi de l'usuari de l'aplicaci G.R.P. 2018
		if(session != null) {
			if(session.getAttribute("usuari") != null) session.removeAttribute("usuari");
			if(session.getAttribute("tipusUsuari") != null) session.removeAttribute("tipusUsuari");
		}
		
		String missatgeInfo = utils.getMsg("msg.info.sortir.aplicacio");
		String redireccio ="/acces/ini";
		
		model.addAttribute("redireccio", redireccio);
		model.addAttribute("missatgeInfo", missatgeInfo);
		
		return "info";
	}


	@PostMapping("/canviContrasenya")
    public String canviContrasenya(Model model, @ModelAttribute("login") String login,  @ModelAttribute("contrasenyaActual") String contrasenyaActual, 
    		@ModelAttribute("contrasenyaNova") String contrasenyaNova, @ModelAttribute("tipusUsuari") String tipusUsuari) {
		
		LOGGER.info("canviContrasenya");
		Boolean contrasenyaValida = false;
		
		
		try {
			//Validar la contrasenya actual proporcionada per l'usuari.
			contrasenyaValida = serveiUsuari.comprovarContrasenya(login, contrasenyaActual);
			
			if(!contrasenyaValida) {

	            model.addAttribute("missatgeError", utils.getMsg("msg.error.contrasenya.nok"));
	            
				return "error";
			}
			
			//Canviar la contrasenya actual per la contrasenya nova.
			serveiUsuari.canviarContrasenya(login, contrasenyaNova);
			
        } catch (ExcepcioUsuari eu) {

        	String missatgeError = utils.getMsg("msg.error.general.bd") + eu.getMessage();
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }
		
		if(tipusUsuari.equalsIgnoreCase("planificador")) 
			model.addAttribute("redireccio", "/menuPlanificador/mostrarMenu");
		
		else if(tipusUsuari.equalsIgnoreCase("tripulant"))
			model.addAttribute("redireccio", "/menuTripulant/mostrarMenu");
		
		model.addAttribute("missatgeInfo", utils.getMsg("msg.info.mod.contrasenya.ok"));
		
		return "info";
	}


	@GetMapping("/tempsSessioExpirat")
    public String tempsSessioExpirat(Model model) {
		
		LOGGER.info("tempsSessioExpirat");
		
		String missatgeInfo = utils.getMsg("msg.info.sessio.expirada");
		String redireccio ="/acces/ini";
		
		model.addAttribute("redireccio", redireccio);
		model.addAttribute("missatgeInfo", missatgeInfo);
		
		return "info";
    }


	@GetMapping("/accesDenegat")
    public String accesDenegat(Model model) {
		
		LOGGER.info("accesDenegat");
		
		String missatgeInfo = utils.getMsg("msg.info.acces.denegat");
		String redireccio ="/acces/ini";
		
		model.addAttribute("redireccio", redireccio);
		model.addAttribute("missatgeInfo", missatgeInfo);
		
		return "info";
    }


}
