package edu.uoc.grp.controller;

import org.springframework.transaction.CannotCreateTransactionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Map;
import edu.uoc.grp.exceptions.ExcepcioPlanificacio;
import edu.uoc.grp.exceptions.ExcepcioInformacio;
import edu.uoc.grp.service.ServeiPlanificacio;
import edu.uoc.grp.service.ServeiInformacio;
import edu.uoc.grp.model.Vaixell;
import edu.uoc.grp.utilities.Utils;


/**
 * Classe ControladorPlanificador.
 * 
 * Classe de tipus controlador encarregada de gestionar les peticions i les respostes relacionades amb les funcionalitats del planificador. 
 *
 * @version 1.0; java version "1.8.0";
 */
@Controller
@RequestMapping("/planificador") 
public class ControladorPlanificador {

	private static final Logger LOGGER = LogManager.getLogger(ControladorPlanificador.class.getName());

    @Autowired
    private ServeiPlanificacio serveiPlanificacio;		

    @Autowired
    private ServeiInformacio serveiInformacio;		

    @Autowired
    private Utils utils;
    

	@PostMapping("/consultarVaixell")
    public String consultarVaixell(Model model, @ModelAttribute("nomVaixell") String nomVaixell) {

		LOGGER.info("consultarVaixell");
		Vaixell vaixell;
		
		LOGGER.debug("nomVaixell: " + nomVaixell);	
		
		Map<String, String> tripulacioMap = new LinkedHashMap<String, String>();
		
        try {

        	//Cercar vaixell. 
        	vaixell = serveiPlanificacio.cercarVaixell(nomVaixell);
        	
        	//Si el vaixell cercat no existeix a dins de la BD, mostrar missatge a l'usuari explicant aquest fet.
        	if (vaixell == null) {

            	String missatgeError = utils.getMsg("msg.error.vaixell.no.trobat.bd");      		
            	LOGGER.error(missatgeError);        		
        		model.addAttribute("missatgeError", missatgeError); 
        		
                return "error";
        	}
        	
        	tripulacioMap = serveiInformacio.obtenirTripulacioVaixell(vaixell.getIdVaixell());

        } catch (ExcepcioPlanificacio ep) {
        	
        	String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
            
        } catch (ExcepcioInformacio ei) {
        	
        	String missatgeError = utils.getMsg("msg.error.general.bd") + ei.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }
        
		model.addAttribute("tripulacioMap", tripulacioMap);        
		model.addAttribute("vaixell", vaixell);

		return "consultaVaixell";
    }	


	@PostMapping("/gestionarAssigAreaVaixell")
    public String gestionarAssigAreaVaixell(Model model, @ModelAttribute("nomVaixell") String nomVaixell) {

		LOGGER.info("gestionarAssigAreaVaixell");
		
    	Map<String, String> areesDisponibles;		
		Vaixell vaixell;
		
        try {

    		//Cercar vaixell.        	
        	vaixell = serveiPlanificacio.cercarVaixell(nomVaixell);
        	
        	//Si el vaixell cercat no existeix a dins de la BD, mostrar missatge a l'usuari explicant aquest fet.
        	if (vaixell == null) {
        		
        		String missatgeError = utils.getMsg("msg.error.vaixell.no.trobat.bd");
            	LOGGER.error(missatgeError);        		
        		model.addAttribute("missatgeError", missatgeError);
        		
                return "error";
        	}
        	
        	//Cercar les arees disponibles (les arees que no estiguin assignades).
        	areesDisponibles = serveiPlanificacio.obtenirAreesDisponibles();
        	
        } catch (ExcepcioPlanificacio ep) {
        	
        	String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }
        
		model.addAttribute("areesDisponibles", areesDisponibles);         
		model.addAttribute("vaixell", vaixell);		
		
		return "assignacioAreaVaixell";
    }		
	
	
	@PostMapping("/guardarAssigAreaVaixell")
    public String guardarAssigAreaVaixell(Model model, @ModelAttribute("idVaixell") Integer idVaixell, @ModelAttribute("idAreaAssigVaixell") Integer idAreaAssigVaixell) {

		LOGGER.info("guardarAssigAreaVaixell");
		
        try {

        	serveiPlanificacio.assignarAreaVaixell(idVaixell, idAreaAssigVaixell);
        	
        } catch (ExcepcioPlanificacio ep) {

            model.addAttribute("redireccio", "/menuPlanificador/mostrarMenu");        	
            model.addAttribute("missatgeError", ep.getMessage());
            return "error";
        }
		
		String missatgeInfo;
		
		if (idAreaAssigVaixell == -1) missatgeInfo = utils.getMsg("msg.info.desfer.assig.area.ok");
		else missatgeInfo = utils.getMsg("msg.info.assig.area.ok");			

		model.addAttribute("missatgeInfo", missatgeInfo);
        model.addAttribute("redireccio", "/menuPlanificador/mostrarMenu");

		return "info";
    }


	@PostMapping("/gestionarAssigMollVaixell")
    public String gestionarAssigMollVaixell(Model model, @ModelAttribute("nomVaixell") String nomVaixell) {

		LOGGER.info("gestionarAssigMollVaixell");
		
    	Map<String, String> mollsDisponibles;		
		Vaixell vaixell;
		
        try {

    		//Cercar vaixell.        	
        	vaixell = serveiPlanificacio.cercarVaixell(nomVaixell);

        	//Si el vaixell cercat no existeix a dins de la BD, mostrar missatge a l'usuari explicant aquest fet.
        	if (vaixell == null) {
        		
        		String missatgeError = utils.getMsg("msg.error.vaixell.no.trobat.bd");
            	LOGGER.error(missatgeError);        		
        		model.addAttribute("missatgeError", missatgeError);
        		
                return "error";
        	}

        	//Cercar els molls disponibles (els molls que no estiguin assignats).
        	mollsDisponibles = serveiPlanificacio.obtenirMollsDisponibles();
        	
        } catch (ExcepcioPlanificacio ep) {
        	
        	String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }
        
		model.addAttribute("mollsDisponibles", mollsDisponibles);         
		model.addAttribute("vaixell", vaixell);		
		
		return "assignacioMollVaixell";
    }		
	
	
	@PostMapping("/guardarAssigMollVaixell")
    public String guardarAssigMollVaixell(Model model, @ModelAttribute("idVaixell") Integer idVaixell, @ModelAttribute("idMollAssigVaixell") Integer idMollAssigVaixell) {

		LOGGER.info("guardarAssigMollVaixell");
		
        try {

        	serveiPlanificacio.assignarMollVaixell(idVaixell, idMollAssigVaixell);
        	
        } catch (ExcepcioPlanificacio ep) {
        	
            model.addAttribute("redireccio", "/menuPlanificador/mostrarMenu");       	
            model.addAttribute("missatgeError", ep.getMessage());
            return "error";
        }
		
		String missatgeInfo;
		
		if (idMollAssigVaixell == -1) missatgeInfo = utils.getMsg("msg.info.desfer.assig.moll.ok");
		else missatgeInfo = utils.getMsg("msg.info.assig.moll.ok");

		model.addAttribute("missatgeInfo", missatgeInfo);
        model.addAttribute("redireccio", "/menuPlanificador/mostrarMenu");

		return "info";
    }	


	@PostMapping("/assignarTripVaixell")
    public String assignarTripVaixell(Model model, @ModelAttribute("nomVaixell") String nomVaixell) {

		LOGGER.info("assignarTripVaixell");
		
		String patro = "", mariner1 = "", mariner2 = "", mariner3 = "", mariner4 = "", mariner5 = "";
		ArrayList<Object> patroList;
		Map<String, String> patronsDisponiblesMap;		
		ArrayList<Map<String, String>> marinersList;
		Map<String, String> marinersDisponiblesMap;		
		Map<String, String> marinersAssigMap;
		Vaixell vaixell;
		
        try {

    		//Cercar vaixell.        	
        	vaixell = serveiPlanificacio.cercarVaixell(nomVaixell);
        	
        	//Si el vaixell cercat no existeix a dins de la BD, mostrar missatge a l'usuari explicant aquest fet.
        	if (vaixell == null) {
        		
            	String missatgeError = utils.getMsg("msg.error.vaixell.no.trobat.bd");
            	LOGGER.error(missatgeError);            	
        		model.addAttribute("missatgeError", missatgeError);
        		
                return "error";
        	}
        	
        	//Comprovar si el vaixell t una tripulaci assignada (si la t la operaci s modificar, altrament s alta).
        	String operacio = serveiPlanificacio.comprovarTripulacioAssigVaixell(vaixell.getIdVaixell()); 
        	
        	//Cercar els patrons disponibles, i si s'escau el patr que ja estigui assignat al vaixell.
        	patroList = serveiPlanificacio.obtenirPatrons(vaixell.getIdVaixell(), operacio);
        	
        	patronsDisponiblesMap = (Map<String, String>) patroList.get(0);
        	patro = (String) patroList.get(1);
        	
        	//Cercar els mariners disponibles, i si s'escau els mariners que ja estiguin assignats al vaixell.
        	marinersList = serveiPlanificacio.obtenirMariners(vaixell.getIdVaixell(), operacio);
        	
        	marinersDisponiblesMap = marinersList.get(0);
            marinersAssigMap = marinersList.get(1);        	
        	
            if (marinersAssigMap.containsKey("MARINER_1")) mariner1 = marinersAssigMap.get("MARINER_1");
            if (marinersAssigMap.containsKey("MARINER_2")) mariner2 = marinersAssigMap.get("MARINER_2");
            if (marinersAssigMap.containsKey("MARINER_3")) mariner3 = marinersAssigMap.get("MARINER_3");            
            if (marinersAssigMap.containsKey("MARINER_4")) mariner4 = marinersAssigMap.get("MARINER_4");            
            if (marinersAssigMap.containsKey("MARINER_5")) mariner5 = marinersAssigMap.get("MARINER_5");            
        	
        } catch (ExcepcioPlanificacio ep) {
        	
        	String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }
        
		model.addAttribute("patro", patro);        
		model.addAttribute("mariner1", mariner1);
		model.addAttribute("mariner2", mariner2);
		model.addAttribute("mariner3", mariner3);		
		model.addAttribute("mariner4", mariner4);		
		model.addAttribute("mariner5", mariner5);		
		model.addAttribute("patronsDisponiblesMap", patronsDisponiblesMap);		
		model.addAttribute("marinersDisponiblesMap", marinersDisponiblesMap);
		model.addAttribute("vaixell", vaixell);		
		
		return "assignacioTripVaixell";
    }	


	@PostMapping("/guardarAssigTripVaixell")
    public String guardarAssigTripVaixell(Model model, @ModelAttribute("idVaixell") Integer idVaixell, 
    		@ModelAttribute("patroSel") Integer patroSel, @ModelAttribute("mariner1Sel") Integer mariner1Sel, 
    		@ModelAttribute("mariner2Sel") Integer mariner2Sel, @ModelAttribute("mariner3Sel") Integer mariner3Sel, 
    		@ModelAttribute("mariner4Sel") Integer mariner4Sel, @ModelAttribute("mariner5Sel") Integer mariner5Sel) {

		LOGGER.info("guardarAssigTripVaixell");
		
		try {
		
			serveiPlanificacio.assignarTripulacioVaixell(idVaixell, patroSel, mariner1Sel, mariner2Sel, mariner3Sel, mariner4Sel, mariner5Sel);

        } catch (ExcepcioPlanificacio ep) {
        	
            model.addAttribute("missatgeError", ep.getMessage());
            return "error";
            
        } catch (CannotCreateTransactionException ccte) {
        	
        	String missatgeError = utils.getMsg("msg.error.general.bd") + ccte.getMessage();
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            return "error";
            
        } catch (Exception e) {
        	
        	String missatgeError = utils.getMsg("msg.error.general") + e.getMessage();
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            return "error";
        }
		
		String  missatgeInfo = utils.getMsg("msg.info.assig.trip.ok");
		model.addAttribute("missatgeInfo", missatgeInfo);
        model.addAttribute("redireccio", "/menuPlanificador/mostrarMenu");

		return "info";
    }


	@GetMapping("/baixaTripulacio")
    public String baixaTripulacio(Model model, @ModelAttribute("idVaixell") Integer idVaixell) {

		LOGGER.info("baixaTripulacio");

        try {

        	serveiPlanificacio.baixaTripulacio(idVaixell);
        	
        } catch (ExcepcioPlanificacio ep) {

        	String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();        	
        	LOGGER.error(missatgeError);
            model.addAttribute("missatgeError", missatgeError);
            
            return "error";
        }
        
		String  missatgeInfo = utils.getMsg("msg.info.baixa.tripulacio.ok");

		model.addAttribute("missatgeInfo", missatgeInfo);
        model.addAttribute("redireccio", "/menuPlanificador/mostrarMenu");

		return "info";
    }
	
	
	
}
