package edu.uoc.grp.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Repository;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import edu.uoc.grp.exceptions.ExcepcioInformacio;
import edu.uoc.grp.exceptions.ExcepcioPlanificacio;
import edu.uoc.grp.mapper.AreaMapper;
import edu.uoc.grp.model.Area;


/**
 * Classe DaoArea.
 * 
 * Classe de tipus repositori que representa l'objecte d'accs a les dades de l'rea que es troben a dins de la base de dades.
 *
 * @version 1.0; java version "1.8.0";
 */
@Repository
public class DaoArea extends JdbcDaoSupport {
	
	private static final Logger LOGGER = LogManager.getLogger(DaoArea.class.getName());	
	
	@Autowired
    public DaoArea(DataSource dataSource) {
        this.setDataSource(dataSource);
    }


    /**
     * Mtode que serveix per a obtenir un llistat de les arees que no estiguin assignades a cap vaixell.
     *
     * @return List<Area> retorna un llistat d'objectes de tipus rea.
     * @throws ExcepcioPlanificacio 
     */
    public List<Area> obtenirAreesDisponibles() throws ExcepcioPlanificacio {
    	
    	List<Area> llistatAreesDisponibles = new ArrayList<Area>();

    	
    	try {
    		
            StringBuffer query = new StringBuffer(); 
            
            query.append("select a.idArea, a.codiArea, a.extensio,");
            query.append(" a.localitzacioLatitud, a.localitzacioLongitud");            
            query.append(" from Area a");            
            query.append(" where a.idArea not in (select ifnull(j.idArea, -1) from Jornada j)");
            query.append(" order by a.codiArea");            

            llistatAreesDisponibles  = getJdbcTemplate().query(query.toString(), new BeanPropertyRowMapper<Area>(Area.class));    		

	    } catch (DataAccessException dae) {
	    	LOGGER.error("obtenirAreesDisponibles: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioPlanificacio(dae.getMessage());
	
	    } catch (Exception e) {
	    	LOGGER.error("obtenirAreesDisponibles: Exception: " + e.getMessage());        	
	        throw new ExcepcioPlanificacio(e.getMessage());
	
	    }    	
    	
    	return llistatAreesDisponibles;
    }


    /**
     * Mtode que serveix per a obtenir una rea a partir del seu identificador.
     *
     * @param idArea objecte que cont el identificador del rea.
     * @return Area retorna un objecte de tipus rea.
     * @throws ExcepcioInformacio 
     */
    public Area cercarArea(Integer idArea)  throws ExcepcioInformacio {
    	
    	Area area = null;
    	
    	try {
	        
	        StringBuffer query = new StringBuffer();
	        
	        query.append("select idArea, codiArea, extensio, localitzacioLatitud, localitzacioLongitud");
            query.append(" from Area");
	        query.append(" where idArea = ? ");
	        
	        AreaMapper mapper = new AreaMapper();
	        area = this.getJdbcTemplate().queryForObject(query.toString(), new Object[] { idArea }, mapper);
        
        } catch (DataAccessException dae) {
        	
        	LOGGER.error("cercarArea: DataAccessException: " + dae.getMessage());
            throw new ExcepcioInformacio(dae.getMessage());
            
        } catch (Exception e) {
        	
        	LOGGER.error("cercarArea: Exception: " + e.getMessage());        	
            throw new ExcepcioInformacio(e.getMessage());
        }
    	
        return area;
    }	    	
   
    
}
