package edu.uoc.grp.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Repository;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import javax.sql.DataSource;
import java.sql.Types;
import edu.uoc.grp.exceptions.ExcepcioPlanificacio;


/**
 * Classe DaoJornada.
 * 
 * Classe de tipus repositori que representa l'objecte d'accs a les dades de la jornada que es troben a dins de la base de dades.
 *
 * @version 1.0; java version "1.8.0";
 */
@Repository
public class DaoJornada extends JdbcDaoSupport {
	
	private static final Logger LOGGER = LogManager.getLogger(DaoJornada.class.getName());	
	
	@Autowired
    public DaoJornada(DataSource dataSource) {
        this.setDataSource(dataSource);
    }

	
    /**
     * Mtode que serveix per a comprovar si existeix una jornada a partir del identificador del vaixell relacionat amb la jornada.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @return Boolean retorna cert si la jornada existeix, altrament retorna fals.
     * @throws ExcepcioPlanificacio 
     */	
    public Boolean jornadaExisteix(Integer idVaixell) throws ExcepcioPlanificacio {
    	
    	Boolean existeixJornada = false;
    	
    	try {
    		
    		StringBuffer query = new StringBuffer();
    	
    		query.append("select count(*) from Jornada where idVaixell = ? ");
	        int countJornada = getJdbcTemplate().queryForObject(query.toString(), new Object[] { idVaixell }, Integer.class);
	        
	        //Si la jornada existeix.
	        if(countJornada == 1) return true;
        
        } catch (DataAccessException dae) {
        	
        	LOGGER.error("jornadaExisteix: DataAccessException: " + dae.getMessage());
            throw new ExcepcioPlanificacio(dae.getMessage());
            
        } catch (Exception e) {
        	
        	LOGGER.error("jornadaExisteix: Exception: " + e.getMessage());        	
            throw new ExcepcioPlanificacio(e.getMessage());
        }
    	
    	return existeixJornada;
    }	


    /**
     * Mtode que serveix per donar d'alta una nova jornada pesquera a l'aplicaci.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @param idMoll objecte que cont el identificador del moll. 
     * @param idArea objecte que cont el identificador del rea. 
     * @throws ExcepcioPlanificacio
     */
    public void altaJornada(Integer idVaixell, Integer idMoll, Integer idArea) throws ExcepcioPlanificacio {
    	
    	try {
    	
	        StringBuffer query = new StringBuffer();    	
	        
            query.append("insert into Jornada (idVaixell, idMoll, idArea)");
            query.append(" VALUES( ?, ?, ? )");
	        
	        Object[] params = { idVaixell, idMoll, idArea };
	        
	        int[] types = { Types.INTEGER, Types.INTEGER, Types.INTEGER };
	        
	        int rows = getJdbcTemplate().update(query.toString(), params, types);        
	        
	    } catch (DataAccessException dae) {
	    	
	    	LOGGER.error("altaJornada: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioPlanificacio(dae.getMessage());
	        
	    } catch (Exception e) {
	    	
	    	LOGGER.error("altaJornada: Exception: " + e.getMessage());
	        throw new ExcepcioPlanificacio(e.getMessage());
	    }   
    }	


    /**
     * Mtode que serveix per a modificar una jornada a partir del identificador del vaixell relacionat amb la jornada.
     *
     * @param campMod objecte que indica el camp que s'ha de modificar.
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @param idMoll objecte que cont el identificador del moll. 
     * @param idArea objecte que cont el identificador del rea. 
     * @throws ExcepcioPlanificacio
     */  
    public void modificaJornada(String campMod, Integer idVaixell, Integer idMoll, Integer idArea) throws ExcepcioPlanificacio {
    	
    	try {
    	
	        StringBuffer query = new StringBuffer();    	
	        
            query.append("update Jornada set");
            if(campMod.equals("MOLL")) query.append(" idMoll = ? ");
            if(campMod.equals("AREA")) query.append(" idArea = ? "); 
            query.append(" where idVaixell = ? ");
            
            Object[] params = null;
            
            if(campMod.equals("MOLL")) params = new Object[]{idMoll, idVaixell};
            if(campMod.equals("AREA")) params = new Object[]{idArea, idVaixell};
	        
	        int[] types = { Types.INTEGER, Types.INTEGER };
	        
	        int rows = getJdbcTemplate().update(query.toString(), params, types);        
	        
	    } catch (DataAccessException dae) {
	    	
	    	LOGGER.error("modificaJornada: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioPlanificacio(dae.getMessage());
	        
	    } catch (Exception e) {
	    	
	    	LOGGER.error("modificaJornada: Exception: " + e.getMessage());
	        throw new ExcepcioPlanificacio(e.getMessage());
	    }   
    }	    
    
}
