package edu.uoc.grp.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Repository;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import edu.uoc.grp.exceptions.ExcepcioPlanificacio;
import edu.uoc.grp.exceptions.ExcepcioInformacio;
import edu.uoc.grp.mapper.MollMapper;
import edu.uoc.grp.model.Moll;


/**
 * Classe DaoMoll.
 * 
 * Classe de tipus repositori que representa l'objecte d'accs a les dades del moll que es troben a dins de la base de dades.
 *
 * @version 1.0; java version "1.8.0";
 */
@Repository
public class DaoMoll extends JdbcDaoSupport {
	
	private static final Logger LOGGER = LogManager.getLogger(DaoMoll.class.getName());	
	
	@Autowired
    public DaoMoll(DataSource dataSource) {
        this.setDataSource(dataSource);
    }
	
	
    /**
     * Mtode que serveix per a obtenir un llistat dels molls que no estiguin assignats a cap vaixell.
     *
     * @return List<Moll> retorna un llistat d'objectes de tipus moll.
     * @throws ExcepcioPlanificacio 
     */
    public List<Moll> obtenirMollsDisponibles() throws ExcepcioPlanificacio {
    	
    	List<Moll> llistatMollsDisponibles = new ArrayList<Moll>();

    	try {
    		
            StringBuffer query = new StringBuffer(); 
            
            query.append("select m.idMoll, m.codiMoll, m.nomPort");
            query.append(" from Moll m");            
            query.append(" where m.idMoll not in (select ifnull(j.idMoll, -1) from Jornada j)");
            query.append(" order by m.codiMoll");            

            llistatMollsDisponibles  = getJdbcTemplate().query(query.toString(), new BeanPropertyRowMapper<Moll>(Moll.class));    		

	    } catch (DataAccessException dae) {
	    	LOGGER.error("obtenirMollsDisponibles: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioPlanificacio(dae.getMessage());
	
	    } catch (Exception e) {
	    	LOGGER.error("obtenirMollsDisponibles: Exception: " + e.getMessage());        	
	        throw new ExcepcioPlanificacio(e.getMessage());
	
	    }    	
    	
    	return llistatMollsDisponibles;
    }


    /**
     * Mtode que serveix per a obtenir un moll a partir del seu identificador.
     *
     * @param idMoll objecte que cont el identificador del moll.
     * @return Moll retorna un objecte de tipus moll.
     * @throws ExcepcioInformacio 
     */
    public Moll cercarMoll(Integer idMoll)  throws ExcepcioInformacio {
    	
    	Moll moll = null;
    	
    	try {
	        
	        StringBuffer query = new StringBuffer();
	        
	        query.append("select idMoll, codiMoll, nomPort");
            query.append(" from Moll");
	        query.append(" where idMoll = ? ");
	        
	        MollMapper mapper = new MollMapper();
	        moll = this.getJdbcTemplate().queryForObject(query.toString(), new Object[] { idMoll }, mapper);
        
        } catch (DataAccessException dae) {
        	
        	LOGGER.error("cercarMoll: DataAccessException: " + dae.getMessage());
            throw new ExcepcioInformacio(dae.getMessage());
            
        } catch (Exception e) {
        	
        	LOGGER.error("cercarMoll: Exception: " + e.getMessage());        	
            throw new ExcepcioInformacio(e.getMessage());
        }
    	
        return moll;
    }	    	
   	
	
}
