package edu.uoc.grp.dao;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Repository;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import java.sql.Types;
import edu.uoc.grp.exceptions.ExcepcioInformacio;
import edu.uoc.grp.exceptions.ExcepcioPlanificacio;
import edu.uoc.grp.mapper.TripulantMapper;
import edu.uoc.grp.model.Tripulant;


/**
 * Classe DaoTripulant.
 * 
 * Classe de tipus repositori que representa l'objecte d'accs a les dades del tripulant que es troben a dins de la base de dades.
 *
 * @version 1.0; java version "1.8.0";
 */
@Repository
public class DaoTripulant extends JdbcDaoSupport {
	
	private static final Logger LOGGER = LogManager.getLogger(DaoTripulant.class.getName());	
	
	@Autowired
    public DaoTripulant(DataSource dataSource) {
        this.setDataSource(dataSource);
    }
	

    /**
     * Mtode que serveix per a cercar un tripulant a partir del nom identificatiu del seu usuari.
     *
     * @param login objecte que cont el nom identificatiu del usuari.
     * @return Tripulant retorna un objecte de tipus tripulant.
     * @throws ExcepcioInformacio 
     */	
    public Tripulant cercarTripulant(String login)  throws ExcepcioInformacio {
    	
    	Tripulant tripulant = null;
    	
    	try {
	        
	        StringBuffer query = new StringBuffer();
	        
	        query.append("select t.idTripulant, t.tipusTripulant, ifnull(t.idVaixell,-1) as vaixellAssignat,");
	        query.append(" concat(e.nom,' ',e.cognom1,' ',e.cognom2) as nomComplet, e.docIdentificatiu");
	        query.append(" from Tripulant t, Empleat e, Usuari u");
	        query.append(" where e.idEmpleat = u.idUsuari");
	        query.append(" and u.idUsuari = t.idTripulant");
	        query.append(" and u.login = ? ");
	        
	        TripulantMapper mapper = new TripulantMapper();
	        tripulant = this.getJdbcTemplate().queryForObject(query.toString(), new Object[] { login }, mapper);
        
        } catch (DataAccessException dae) {
        	
        	LOGGER.error("cercarTripulant: DataAccessException: " + dae.getMessage());
            throw new ExcepcioInformacio(dae.getMessage());
            
        } catch (Exception e) {
        	
        	LOGGER.error("cercarTripulant: Exception: " + e.getMessage());        	
            throw new ExcepcioInformacio(e.getMessage());
        }
    	
        return tripulant;
    }	    		

	
    /**
     * Mtode que serveix per a obtenir la tripulaci d'un vaixell en concret a partir del seu identificador.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @return List<Tripulant> retorna un llistat d'objectes de tipus tripulant.
     * @throws ExcepcioInformacio 
     */    
    public List<Tripulant> obtenirTripulacioVaixell(Integer idVaixell) throws ExcepcioInformacio {
    	
        List<Tripulant> llistatTripulant = new ArrayList<Tripulant>();

    	try {
    		
            StringBuffer query = new StringBuffer(); 
            
            query.append("select t.idTripulant, t.tipusTripulant, t.idVaixell as vaixellAssignat,");
            query.append(" concat(e.nom,' ',e.cognom1,' ',e.cognom2) as nomComplet");            
            query.append(" from Tripulant t, Empleat e");
            query.append(" where t.idTripulant = e.idEmpleat");
            query.append(" and t.idVaixell = ? ");
            query.append(" order by e.nom, e.cognom1, e.cognom2");
            
            llistatTripulant  = getJdbcTemplate().query(query.toString(), new Object[] { idVaixell }, new BeanPropertyRowMapper<Tripulant>(Tripulant.class));    		

	    } catch (DataAccessException dae) {
	    	
	    	LOGGER.error("obtenirTripulacioVaixell: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioInformacio(dae.getMessage());
	
	    } catch (Exception e) {
	    	
	    	LOGGER.error("obtenirTripulacioVaixell: Exception: " + e.getMessage());        	
	        throw new ExcepcioInformacio(e.getMessage());
	    }
        
    	return llistatTripulant;
    }	


    /**
     * Mtode que serveix per a comprovar si un vaixell t una tripulaci assignada.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @return Boolean retorna cert si el vaixell t tripulaci, altrament retorna fals.
     * @throws ExcepcioPlanificacio 
     */    
    public Boolean tripulacioAssigVaixell(Integer idVaixell)  throws ExcepcioPlanificacio {
    	
    	Boolean tripAssigVaixell = false;
    	
    	try {
    		
    		StringBuffer query = new StringBuffer();
    	
    		query.append("select count(*) from Tripulant where idVaixell = ? ");
	        int countVaixell = getJdbcTemplate().queryForObject(query.toString(), new Object[] { idVaixell }, Integer.class);
	        
	        //Si el vaixell t tripulaci assignada.
	        if(countVaixell > 0) return true;
        
        } catch (DataAccessException dae) {
        	
        	LOGGER.error("tripulacioAssigVaixell: DataAccessException: " + dae.getMessage());
            throw new ExcepcioPlanificacio(dae.getMessage());
            
        } catch (Exception e) {
        	
        	LOGGER.error("tripulacioAssigVaixell: Exception: " + e.getMessage());        	
            throw new ExcepcioPlanificacio(e.getMessage());
        }
    	
    	return tripAssigVaixell;
    }	    
 

    /**
     * Mtode que serveix per a obtenir els tripulants disponibles, i tamb la tripulaci d'un vaixell en concret a partir del seu identificador.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @param tipusTripulant objecte que cont el tipus de tripulant. 
     * @return List<Tripulant> retorna un llistat d'objectes de tipus tripulant.
     * @throws ExcepcioPlanificacio 
     */    
    public List<Tripulant> obtenirTripDisponiblesAlta(Integer idVaixell, String tipusTripulant) throws ExcepcioPlanificacio {
    	
    	LOGGER.info("obtenirTripDisponiblesAlta");  	
        List<Tripulant> llistatTripulant = new ArrayList<Tripulant>();

    	try {
    		
            StringBuffer query = new StringBuffer(); 
            
            query.append("select t.idTripulant, t.tipusTripulant, t.idVaixell as vaixellAssignat,");
            query.append(" concat(e.nom,' ',e.cognom1,' ',e.cognom2) as nomComplet");            
            query.append(" from Tripulant t, Empleat e");
            query.append(" where t.idTripulant = e.idEmpleat");
            query.append(" and t.tipusTripulant = ? ");
            query.append(" and (t.idVaixell is null or t.idVaixell = ? )");
            query.append(" and 'A' = (select u.estatEnSistema from Usuari u where u.idUsuari = e.idEmpleat)");            
            query.append(" order by e.nom, e.cognom1, e.cognom2");            
            
            llistatTripulant  = getJdbcTemplate().query(query.toString(), new Object[] { tipusTripulant, idVaixell }, new BeanPropertyRowMapper<Tripulant>(Tripulant.class));    		

	    } catch (DataAccessException dae) {
	    	
	    	LOGGER.error("obtenirTripDisponiblesAlta: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioPlanificacio(dae.getMessage());
	
	    } catch (Exception e) {
	    	
	    	LOGGER.error("obtenirTripDisponiblesAlta: Exception: " + e.getMessage());        	
	        throw new ExcepcioPlanificacio(e.getMessage());
	    }
        
    	return llistatTripulant;
    }	


    /**
     * Mtode que serveix per a obtenir els tripulants disponibles, i tamb la tripulaci d'un vaixell en concret a partir del seu identificador.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @param tipusTripulant objecte que cont el tipus de tripulant. 
     * @return List<Tripulant> retorna un llistat d'objectes de tipus tripulant.
     * @throws ExcepcioPlanificacio 
     */    
    public List<Tripulant> obtenirTripDisponiblesModificar(Integer idVaixell, String tipusTripulant) throws ExcepcioPlanificacio {
    	
    	LOGGER.info("obtenirTripDisponiblesModificar");  	
        List<Tripulant> llistatTripulant = new ArrayList<Tripulant>();

    	try {
    		
            StringBuffer query = new StringBuffer(); 
            
            query.append("select t.idTripulant, t.tipusTripulant, t.idVaixell as vaixellAssignat,");
            query.append(" concat(e.nom,' ',e.cognom1,' ',e.cognom2) as nomComplet");            
            query.append(" from Tripulant t, Empleat e");
            query.append(" where t.idTripulant = e.idEmpleat");
            query.append(" and t.tipusTripulant = ? ");
            query.append(" and (t.idVaixell is null or t.idVaixell = ? )");
            query.append(" and ('A' = (select u.estatEnSistema from Usuari u where u.idUsuari = e.idEmpleat)");
            query.append(" or ('B' = (select u.estatEnSistema from Usuari u where u.idUsuari = e.idEmpleat) and t.idVaixell = ? ))");          
            query.append(" order by e.nom, e.cognom1, e.cognom2");       
            
            
            llistatTripulant  = getJdbcTemplate().query(query.toString(), new Object[] { tipusTripulant, idVaixell, idVaixell }, new BeanPropertyRowMapper<Tripulant>(Tripulant.class));    		

	    } catch (DataAccessException dae) {
	    	
	    	LOGGER.error("obtenirTripDisponiblesModificar: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioPlanificacio(dae.getMessage());
	
	    } catch (Exception e) {
	    	
	    	LOGGER.error("obtenirTripDisponiblesModificar: Exception: " + e.getMessage());        	
	        throw new ExcepcioPlanificacio(e.getMessage());
	    }
        
    	return llistatTripulant;
    }    


    /**
     * Mtode que serveix per a assignar o modificar la assignaci d'una tripulaci a un vaixell a partir del seu identificador.
     * Les operacions d'aquest mtode formen una nica transacci, a fi de protegir la integritat de les dades dins la Base de Dades.
     * Si es produeix algun error es llanar una excepci i la transacci quedar desfeta (rollback).
     * Sin es produeix un error les operacions de la transacci es faran efectives (commit).   
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @param tripulacio llistat de identificadors dels tripulants del vaixell. 
     * @throws ExcepcioPlanificacio
     */      
    @Transactional(rollbackFor={ExcepcioPlanificacio.class})
    public void actualitzaTripulant(Integer idVaixell, List<Integer> tripulacio) throws ExcepcioPlanificacio {
    	
    	try {
    		
	        StringBuffer queryUpdate = new StringBuffer();
	        StringBuffer querySelect = new StringBuffer();
	        
	        //Treure els tripulants anteriorment assignats al vaixell. 
	        queryUpdate.append("update Tripulant set ");
	        queryUpdate.append(" idVaixell = null ");
	        queryUpdate.append(" where idVaixell = ? ");	
	        
	        int rows = getJdbcTemplate().update(queryUpdate.toString(), new Object[]{ idVaixell }, new int[]{ Types.INTEGER });   	        

	    	LOGGER.debug("actualitzaTripulant: tripulants trets: " + rows);
	    	
	    	//Comprovar que el tripulant no estigui assignat a un altre vaixell, fet que noms
	    	//podria passar si es realitzen planificacions concurrents.
	    	querySelect = new StringBuffer();
	    	querySelect.append("select ifnull(idVaixell,-1)");
	    	querySelect.append(" from Tripulant");
	    	querySelect.append(" where idTripulant = ? ");

	        //Assignaci de la nova tripulaci al vaixell.	    	
	    	queryUpdate = new StringBuffer();  
	    	queryUpdate.append("update Tripulant set ");
	    	queryUpdate.append(" idVaixell = ? ");
	    	queryUpdate.append(" where idTripulant = ? ");

	        for(Integer idTripulant : tripulacio) {
	        	
	        	int vaixellAssignat = getJdbcTemplate().queryForObject(querySelect.toString(), new Object[] { idTripulant }, Integer.class);
	        	
	        	if(vaixellAssignat != -1) throw new ExcepcioPlanificacio("CONCURRENCY_ERROR");
	        	
		        rows = getJdbcTemplate().update(queryUpdate.toString(), new Object[]{ idVaixell, idTripulant }, new int[]{ Types.INTEGER, Types.INTEGER }); 
		        
	        }
        
	    } catch (DataAccessException dae) {
	    	
	    	LOGGER.error("actualitzaTripulant: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioPlanificacio(dae.getMessage());
	        
	    } catch (Exception e) {
	    	
	    	LOGGER.error("actualitzaTripulant: Exception: " + e.getMessage());
	        throw new ExcepcioPlanificacio(e.getMessage());
	    }  
    }      


    /**
     * Mtode que serveix per a donar de baixa la tripulaci de un vaixell a partir del seu identificador.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @throws ExcepcioPlanificacio
     */    
    public void baixaTripulacio(Integer idVaixell) throws ExcepcioPlanificacio {
    	
    	try {
    		
	        StringBuffer query = new StringBuffer();
	        query.append("update Tripulant set ");
	        query.append(" idVaixell = null ");
	        query.append(" where idVaixell = ? ");	
	        
	        int rows = getJdbcTemplate().update(query.toString(), new Object[]{ idVaixell }, new int[]{ Types.INTEGER });   	        
        
	    } catch (DataAccessException dae) {
	    	
	    	LOGGER.error("baixaTripulacio: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioPlanificacio(dae.getMessage());
	        
	    } catch (Exception e) {
	    	
	    	LOGGER.error("baixaTripulacio: Exception: " + e.getMessage());
	        throw new ExcepcioPlanificacio(e.getMessage());
	    }   
    }	        


    
}
