package edu.uoc.grp.dao;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Repository;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import javax.sql.DataSource;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;
import edu.uoc.grp.exceptions.ExcepcioUsuari;
import edu.uoc.grp.mapper.UsuariMapper;
import edu.uoc.grp.mapper.EmpleatMapper;
import edu.uoc.grp.model.Empleat;
import edu.uoc.grp.model.Usuari;


/**
 * Classe DaoUsuari.
 * 
 * Classe de tipus repositori que representa l'objecte d'accs a les dades del usuari que es troben a dins de la base de dades.
 *
 * @version 1.0; java version "1.8.0";
 */
@Repository
public class DaoUsuari extends JdbcDaoSupport {
	
	private static final Logger LOGGER = LogManager.getLogger(DaoUsuari.class.getName());	
	
	@Autowired
    public DaoUsuari(DataSource dataSource) {
        this.setDataSource(dataSource);
    }


    /**
     * Mtode que serveix per a comprovar si existeix un usuari a partir del seu nom identificatiu.
     *
     * @param login objecte que cont el nom identificatiu del usuari.
     * @return Boolean retorna cert si l'usuari existeix, altrament retorna fals.
     * @throws ExcepcioUsuari 
     */    
    public Boolean usuariExisteix(String login)  throws ExcepcioUsuari {
    	
    	Boolean existeixUsuari = false;
    	
    	try {
    		
    		StringBuffer query = new StringBuffer();
    	
    		query.append("select count(*) from Usuari where login = ? ");
	        int countUsuari = getJdbcTemplate().queryForObject(query.toString(), new Object[] { login }, Integer.class);
	        
	        //Si l'usuari existeix.
	        if(countUsuari == 1) return true;
        
        } catch (DataAccessException dae) {
        	LOGGER.error("usuariExisteix: DataAccessException: " + dae.getMessage());
            throw new ExcepcioUsuari(dae.getMessage());
        } catch (Exception e) {
        	LOGGER.error("usuariExisteix: Exception: " + e.getMessage());        	
            throw new ExcepcioUsuari(e.getMessage());
        }
    	
    	return existeixUsuari;
    }	


    /**
     * Mtode que serveix per a cercar un usuari a partir del seu identificador.
     *
     * @param login objecte que cont el nom identificatiu del usuari.
     * @return Usuari retorna un objecte de tipus usuari.
     * @throws ExcepcioUsuari 
     */    
    public Usuari cercarUsuari(String login)  throws ExcepcioUsuari {
    	
    	Usuari usuari = null;
    	
    	try {
	        
	        StringBuffer query = new StringBuffer();
	        
	        query.append("select idUsuari, tipusUsuari, login, contrasenya, email, estatEnSistema,");
	        query.append(" usuariAlta, dataAlta, usuariModificacio, dataModificacio, usuariBaixa, dataBaixa");	        
	        query.append(" from Usuari");
	        query.append(" where login = ? ");
	        UsuariMapper mapper = new UsuariMapper();
	        usuari = this.getJdbcTemplate().queryForObject(query.toString(), new Object[] { login }, mapper);
        
        } catch (DataAccessException dae) {
        	LOGGER.error("cercarUsuari: DataAccessException: " + dae.getMessage());
            throw new ExcepcioUsuari(dae.getMessage());
        } catch (Exception e) {
        	LOGGER.error("cercarUsuari: Exception: " + e.getMessage());        	
            throw new ExcepcioUsuari(e.getMessage());
        }
    	
        return usuari;
    }	    


    /**
     * Mtode que serveix per cercar a un empleat a partir del seu identificador.
     *
     * @param idEmpleat objecte que cont el identificador del empleat.
     * @return Empleat retorna un objecte de tipus empleat.
     * @throws ExcepcioUsuari 
     */    
    public Empleat cercarEmpleat(Integer idEmpleat)  throws ExcepcioUsuari {
    	
    	Empleat empleat = null;
    	
    	try {
	        
	        StringBuffer query = new StringBuffer();
	        
	        query.append("select idEmpleat, tipusEmpleat, docIdentificatiu, nom, cognom1, cognom2, telefon, adreca, ciutat, pais");
	        query.append(" from Empleat");
	        query.append(" where idEmpleat = ? ");
	        EmpleatMapper mapper = new EmpleatMapper();
	        empleat = this.getJdbcTemplate().queryForObject(query.toString(), new Object[] { idEmpleat }, mapper);
        
        } catch (DataAccessException dae) {
        	LOGGER.error("cercarEmpleat: DataAccessException: " + dae.getMessage());
            throw new ExcepcioUsuari(dae.getMessage());
        } catch (Exception e) {
        	LOGGER.error("cercarEmpleat: Exception: " + e.getMessage());        	
            throw new ExcepcioUsuari(e.getMessage());
        }
    	
        return empleat;
    }	    


    /**
     * Mtode que serveix per a obtenir un llistat dels usuaris de l'aplicaci.
     *
     * @return List<Usuari> retorna un llistat d'objectes de tipus usuari.
     * @throws ExcepcioUsuari 
     */    
    public List<Usuari> obtenirLlistatUsuaris() throws ExcepcioUsuari {
    	
        List<Usuari> llistatUsuari = new ArrayList<Usuari>();

    	try {
    		
            StringBuffer query = new StringBuffer();
            
            query.append("select u.idUsuari, u.tipusUsuari, u.login, u.contrasenya, e.docIdentificatiu,");
            query.append(" concat(e.nom,' ',e.cognom1,' ',e.cognom2) as nomComplet, u.email, u.estatEnSistema");
            query.append(" from Usuari u, Empleat e");
            query.append(" where u.idUsuari = e.idEmpleat");
            query.append(" order by login, e.nom, cognom1, cognom2");

            llistatUsuari  = getJdbcTemplate().query(query.toString(), new BeanPropertyRowMapper<Usuari>(Usuari.class));    		

	    } catch (DataAccessException dae) {
	    	LOGGER.error("obtenirLlistatUsuaris: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioUsuari(dae.getMessage());
	    } catch (Exception e) {
	    	LOGGER.error("obtenirLlistatUsuaris: Exception: " + e.getMessage());
	        throw new ExcepcioUsuari(e.getMessage());
	    }
        
    	return llistatUsuari;
    }


    /**
     * Mtode que serveix per a obtenir un llistat dels empleats de l'empresa.
     *
     * @return List<Empleat> retorna un llistat d'objectes de tipus empleat.
     * @throws ExcepcioUsuari 
     */        
    public List<Empleat> obtenirLlistatEmpleats() throws ExcepcioUsuari {
    	
        List<Empleat> llistatEmpleat = new ArrayList<Empleat>();

    	try {
    		
            StringBuffer query = new StringBuffer();

            query.append("select idEmpleat, docIdentificatiu, nom, cognom1, cognom2, tipusEmpleat");
            query.append(" from Empleat");
            query.append(" where tipusEmpleat IN ('ADMINISTRADOR','PLANIFICADOR','TRIPULANT')");
            query.append(" and idEmpleat NOT IN (select idUsuari from Usuari)");
            query.append(" order by nom, cognom1, cognom2");            
            
            llistatEmpleat  = getJdbcTemplate().query(query.toString(), new BeanPropertyRowMapper<Empleat>(Empleat.class));    		

	    } catch (DataAccessException dae) {
	    	LOGGER.error("obtenirLlistatEmpleats: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioUsuari(dae.getMessage());
	    } catch (Exception e) {
	    	LOGGER.error("obtenirLlistatEmpleats: Exception: " + e.getMessage());
	        throw new ExcepcioUsuari(e.getMessage());
	    }
        
    	return llistatEmpleat;
    }
    
    
    /**
     * Mtode que serveix per modificar el estat (actiu o baixa) de un usuari en concret.
     *
     * @param usuariAdministrador objecte que cont el nom identificatiu del usuari administrador.
     * @param login objecte que cont el nom identificatiu del usuari.
     * @param estatUsuari objecte que cont el estat del usuari.
     * @throws ExcepcioUsuari
     */			    
    public void actualitzaEstatUsuari(String usuariAdministrador, String login, String estatUsuari) throws ExcepcioUsuari {
    	
    	try {
    	
	        StringBuffer query = new StringBuffer();
	        
	        query.append("update Usuari set estatEnSistema = ?,");
	
	        if ("A".equalsIgnoreCase(estatUsuari))
	            query.append(" usuariAlta = ?, dataAlta = CURRENT_DATE");
	
	        else if ("B".equalsIgnoreCase(estatUsuari))
	            query.append(" usuariBaixa = ?, dataBaixa = CURRENT_DATE");
	
	        query.append(" where login = ? ");    
	        

	        Object[] params = { estatUsuari, usuariAdministrador, login };
	        
	        int[] types = { Types.VARCHAR, Types.VARCHAR, Types.VARCHAR };
	        
	        int rows = getJdbcTemplate().update(query.toString(), params, types);
        
	    } catch (DataAccessException dae) {
	    	LOGGER.error("actualitzaEstatUsuari: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioUsuari(dae.getMessage());
	    } catch (Exception e) {
	    	LOGGER.error("actualitzaEstatUsuari: Exception: " + e.getMessage());
	        throw new ExcepcioUsuari(e.getMessage());
	    }    
    }
    
    
    /**
     * Mtode que serveix per modificar les dades de un usuari en concret.
     *
     * @param usuariAdministrador objecte que cont el nom identificatiu del usuari administrador.
     * @param idUsuari objecte que cont el identificador del usuari.
     * @param login objecte que cont el nom identificatiu del usuari.
     * @param contrasenya objecte que cont la contrasenya del usuari.
     * @param eMail objecte que cont el correu electrnic del usuari. 
     * @throws ExcepcioUsuari
     */	    
    public void modificaUsuari(String usuariAdministrador, Integer idUsuari, String login, String contrasenya, String eMail) throws ExcepcioUsuari {
    	
    	try {
    	
	        StringBuffer query = new StringBuffer();    	
	        
	        query.append("update Usuari set login = ?, contrasenya = ?, eMail = ?,");
	        query.append(" usuariModificacio = ?, dataModificacio = CURRENT_DATE");
	        query.append(" where idUsuari = ? ");    
	        
	        Object[] params = { login, contrasenya, eMail, usuariAdministrador, idUsuari };
	        
	        int[] types = { Types.VARCHAR, Types.VARCHAR, Types.VARCHAR, Types.VARCHAR, Types.INTEGER };
	        
	        int rows = getJdbcTemplate().update(query.toString(), params, types);        
	        
	    } catch (DataAccessException dae) {
	    	LOGGER.error("modificaUsuari: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioUsuari(dae.getMessage());
	    } catch (Exception e) {
	    	LOGGER.error("modificaUsuari: Exception: " + e.getMessage());
	        throw new ExcepcioUsuari(e.getMessage());
	    }   
    }


    /**
     * Mtode que serveix per donar de alta a un nou usuari de l'aplicaci.
     *
     * @param usuariAdministrador objecte que cont el nom identificatiu del usuari administrador.
     * @param idUsuari objecte que cont el identificador del usuari.
     * @param tipusUsuari objecte que cont el tipus de usuari. 
     * @param login objecte que cont el nom identificatiu del usuari.
     * @param contrasenya objecte que cont la contrasenya del usuari.
     * @param eMail objecte que cont el correu electrnic del usuari. 
     * @throws ExcepcioUsuari
     */		
    public void altaUsuari(String usuariAdministrador, Integer idUsuari, String tipusUsuari, String login, String contrasenya, String eMail) throws ExcepcioUsuari {
    	
    	try {
    	
	        StringBuffer query = new StringBuffer();    	
	        
            query.append("insert into Usuari (idUsuari, tipusUsuari, login, contrasenya, email, estatEnSistema, dataAlta, usuariAlta)");
            query.append(" VALUES( ?, ?, ?, ?, ?, 'A', CURRENT_DATE, ? )");
	        
	        Object[] params = { idUsuari, tipusUsuari, login, contrasenya, eMail, usuariAdministrador };
	        
	        int[] types = { Types.INTEGER, Types.VARCHAR, Types.VARCHAR, Types.VARCHAR, Types.VARCHAR, Types.VARCHAR };
	        
	        int rows = getJdbcTemplate().update(query.toString(), params, types);        
	        
	    } catch (DataAccessException dae) {
	    	LOGGER.error("altaUsuari: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioUsuari(dae.getMessage());
	    } catch (Exception e) {
	    	LOGGER.error("altaUsuari: Exception: " + e.getMessage());
	        throw new ExcepcioUsuari(e.getMessage());
	    }   
    }


    /**
     * Mtode que serveix per canviar la contrasenya de un usuari en concret a partir del seu nom identificatiu de usuari.
     *
     * @param usuariModificacio objecte que cont el nom identificatiu del usuari administrador.
     * @param login objecte que cont el nom identificatiu del usuari.
     * @param contrasenya objecte que cont la contrasenya del usuari.
     * @throws ExcepcioUsuari
     */		
    public void canviarContrasenya(String usuariModificacio, String login, String contrasenya) throws ExcepcioUsuari {
    	
    	try {
    	
	        StringBuffer query = new StringBuffer();    	
	        
	        query.append("update Usuari set");
	        query.append(" contrasenya = ?, usuariModificacio = ?,");	        
	        query.append(" dataModificacio = CURRENT_DATE");
	        query.append(" where login = ? ");    
   
	        
	        Object[] params = { contrasenya, usuariModificacio, login };
	        
	        int[] types = { Types.VARCHAR, Types.VARCHAR, Types.VARCHAR };
	        
	        int rows = getJdbcTemplate().update(query.toString(), params, types);        
	        
	    } catch (DataAccessException dae) {
	    	
	    	LOGGER.error("canviarContrasenya: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioUsuari(dae.getMessage());
	        
	    } catch (Exception e) {
	    	
	    	LOGGER.error("canviarContrasenya: Exception: " + e.getMessage());
	        throw new ExcepcioUsuari(e.getMessage());
	    }   
    }    
    
    
}
