package edu.uoc.grp.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Repository;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import javax.sql.DataSource;
import java.util.ArrayList;
import java.util.List;
import edu.uoc.grp.exceptions.ExcepcioPlanificacio;
import edu.uoc.grp.exceptions.ExcepcioInformacio;
import edu.uoc.grp.mapper.VaixellMapper;
import edu.uoc.grp.model.Vaixell;


/**
 * Classe DaoVaixell.
 * 
 * Classe de tipus repositori que representa l'objecte d'accs a les dades del vaixell que es troben a dins de la base de dades.
 *
 * @version 1.0; java version "1.8.0";
 */
@Repository
public class DaoVaixell extends JdbcDaoSupport {
	
	private static final Logger LOGGER = LogManager.getLogger(DaoVaixell.class.getName());	
	
	@Autowired
    public DaoVaixell(DataSource dataSource) {
        this.setDataSource(dataSource);
    }


    /**
     * Mtode que serveix per a comprovar si existeix un vaixell a partir del seu nom.
     *
     * @param nomVaixell objecte que cont el nom del vaixell.
     * @return Boolean retorna cert si el vaixell existeix, altrament retorna fals.
     * @throws ExcepcioPlanificacio 
     */    
    public Boolean vaixellExisteix(String nomVaixell)  throws ExcepcioPlanificacio {
    	
    	Boolean existeixVaixell = false;
    	
    	try {
    		
    		StringBuffer query = new StringBuffer();
    	
    		query.append("select count(*) from Vaixell where nomVaixell = ? ");
	        int countVaixell = getJdbcTemplate().queryForObject(query.toString(), new Object[] { nomVaixell }, Integer.class);
	        
	        //Si el vaixell existeix.
	        if(countVaixell == 1) return true;
        
        } catch (DataAccessException dae) {
        	
        	LOGGER.error("vaixellExisteix: DataAccessException: " + dae.getMessage());
            throw new ExcepcioPlanificacio(dae.getMessage());
            
        } catch (Exception e) {
        	
        	LOGGER.error("vaixellExisteix: Exception: " + e.getMessage());        	
            throw new ExcepcioPlanificacio(e.getMessage());
        }
    	
    	return existeixVaixell;
    }	


    /**
     * Mtode que serveix per a obtenir un llistat dels vaixells.
     *
     * @return List<Vaixell> retorna un llistat d'objectes de tipus vaixell.
     * @throws ExcepcioPlanificacio 
     */   	
    public List<Vaixell> obtenirLlistatVaixells() throws ExcepcioPlanificacio {
    	
        List<Vaixell> llistatVaixell = new ArrayList<Vaixell>();

    	try {
    		
            StringBuffer query = new StringBuffer(); 
            
            query.append("select v.idVaixell, v.nomVaixell, v.tipusVaixell, v.eslora,");
            query.append(" a.codiArea as areaAssignada, m.codiMoll as mollAssignat,");
            query.append(" (select if(count(*)>0,'true','false') from tripulant t where v.idVaixell = t.idVaixell) as tripAssignada");            
            query.append(" from Vaixell v left outer join Jornada j on v.idVaixell = j.idVaixell");            
            query.append(" left outer join Area a on j.idArea = a.idArea");
            query.append(" left outer join Moll m on j.idMoll = m.idMoll");            
            query.append(" order by v.idVaixell");

            llistatVaixell  = getJdbcTemplate().query(query.toString(), new BeanPropertyRowMapper<Vaixell>(Vaixell.class));    		

	    } catch (DataAccessException dae) {
	    	
	    	LOGGER.error("obtenirLlistatVaixells: DataAccessException: " + dae.getMessage());
	        throw new ExcepcioPlanificacio(dae.getMessage());
	
	    } catch (Exception e) {
	    	
	    	LOGGER.error("obtenirLlistatVaixells: Exception: " + e.getMessage());        	
	        throw new ExcepcioPlanificacio(e.getMessage());
	    }
        
    	return llistatVaixell;
    }


    /**
     * Mtode que serveix per a cercar un vaixell a partir del seu nom.
     *
     * @param nomVaixell objecte que cont el nom del vaixell.
     * @return Vaixell retorna un objecte de tipus vaixell.
     * @throws ExcepcioPlanificacio 
     */    
    public Vaixell cercarVaixell(String nomVaixell)  throws ExcepcioPlanificacio {
    	
    	Vaixell vaixell = null;
    	
    	try {
	        
	        StringBuffer query = new StringBuffer();
	        
	        query.append("select v.idVaixell, v.nomVaixell, v.tipusVaixell, v.eslora,");
            query.append(" a.codiArea as areaAssignada, ifnull(a.idArea, -1) as idAreaAssignada,");	        
            query.append(" m.codiMoll as mollAssignat, ifnull(m.idMoll, -1) as idMollAssignat");	        
            query.append(" from Vaixell v left outer join Jornada j on v.idVaixell = j.idVaixell");
            query.append(" left outer join Area a on j.idArea = a.idArea");            
            query.append(" left outer join Moll m on j.idMoll = m.idMoll");            
	        query.append(" where v.nomVaixell = ? ");
	        
	        VaixellMapper mapper = new VaixellMapper();
	        vaixell = this.getJdbcTemplate().queryForObject(query.toString(), new Object[] { nomVaixell }, mapper);
        
        } catch (DataAccessException dae) {
        	
        	LOGGER.error("cercarVaixell: DataAccessException: " + dae.getMessage());
            throw new ExcepcioPlanificacio(dae.getMessage());
            
        } catch (Exception e) {
        	
        	LOGGER.error("cercarVaixell: Exception: " + e.getMessage());        	
            throw new ExcepcioPlanificacio(e.getMessage());
        }
    	
        return vaixell;
    }	    	


    /**
     * Mtode que serveix per a cercar un vaixell a partir del seu identificador.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @return Vaixell retorna un objecte de tipus vaixell.
     * @throws ExcepcioInformacio 
     */    
    public Vaixell cercarVaixell(Integer idVaixell)  throws ExcepcioInformacio {
    	
    	Vaixell vaixell = null;
    	
    	try {
	        
	        StringBuffer query = new StringBuffer();
	        
	        query.append("select v.idVaixell, v.nomVaixell, v.tipusVaixell, v.eslora,");
            query.append(" a.codiArea as areaAssignada, ifnull(a.idArea, -1) as idAreaAssignada,");	        
            query.append(" m.codiMoll as mollAssignat, ifnull(m.idMoll, -1) as idMollAssignat");	        
            query.append(" from Vaixell v left outer join Jornada j on v.idVaixell = j.idVaixell");
            query.append(" left outer join Area a on j.idArea = a.idArea");            
            query.append(" left outer join Moll m on j.idMoll = m.idMoll");            
	        query.append(" where v.idVaixell = ? ");
	        
	        VaixellMapper mapper = new VaixellMapper();
	        vaixell = this.getJdbcTemplate().queryForObject(query.toString(), new Object[] { idVaixell }, mapper);
        
        } catch (DataAccessException dae) {
        	
        	LOGGER.error("cercarVaixell: DataAccessException: " + dae.getMessage());
            throw new ExcepcioInformacio(dae.getMessage());
            
        } catch (Exception e) {
        	
        	LOGGER.error("cercarVaixell: Exception: " + e.getMessage());        	
            throw new ExcepcioInformacio(e.getMessage());
        }
    	
        return vaixell;
    }	    	

    
}
