package edu.uoc.grp.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import edu.uoc.grp.exceptions.ExcepcioInformacio;
import edu.uoc.grp.dao.DaoTripulant;
import edu.uoc.grp.dao.DaoVaixell;
import edu.uoc.grp.dao.DaoArea;
import edu.uoc.grp.dao.DaoMoll;
import edu.uoc.grp.model.Tripulant;
import edu.uoc.grp.model.Vaixell;
import edu.uoc.grp.model.Area;
import edu.uoc.grp.model.Moll;


/**
 * Classe ServeiInformacio.
 * 
 * Classe de tipus servei que proporciona els mtodes necessaris per a gestionar la informaci sobre les jornades pesqueres. 
 *
 * @version 1.0; java version "1.8.0";
 */
@Service("ServeiInformacio")
public class ServeiInformacio {
	
	private static final Logger LOGGER = LogManager.getLogger(ServeiInformacio.class.getName());	

	@Autowired	
	private DaoTripulant daoTripulant;
	
	@Autowired
	private DaoVaixell daoVaixell;

	@Autowired
	private DaoArea daoArea;

	@Autowired
	private DaoMoll daoMoll;
	

    /**
     * Mtode que serveix per obtenir la tripulaci d'un vaixell en concret a partir del seu indentificador.
     *
     * @param idVaixell objecte que cont l'indentificador del vaixell.
     * @return Map<String, String> retorna un mapa amb els objectes que contenen la tripulaci d'un vaixell.
     * @throws ExcepcioInformacio
     */	
	public Map<String, String> obtenirTripulacioVaixell(Integer idVaixell) throws ExcepcioInformacio {

		LOGGER.info("obtenirTripulacioVaixell");		
		
		List<Tripulant> llistatTripulant = daoTripulant.obtenirTripulacioVaixell(idVaixell);
		
		Map<String, String> tripulacioMap = new LinkedHashMap<String, String>();
		
        int i = 1;
        for (Tripulant tripulant : llistatTripulant) {

            if ("PATRO".equalsIgnoreCase(tripulant.getTipusTripulant())) {
                tripulacioMap.put("PATRO", tripulant.getNomComplet());
            }

            if ("MARINER".equalsIgnoreCase(tripulant.getTipusTripulant())) {
                tripulacioMap.put("MARINER_" + i, tripulant.getNomComplet());
                i++;
            }
        }

        return tripulacioMap;		
	}


    /**
     * Mtode que serveix per a cercar les dades del tripulant del qual es passa el seu nom identificatiu d'usuari com a parmetre.
     *
     * @param login objecte que cont el nom identificatiu del usuari.
     * @return Tripulant retorna un tripulant.
     * @throws ExcepcioInformacio
     */	
	public Tripulant cercarTripulant(String login) throws ExcepcioInformacio {

		LOGGER.info("cercarTripulant");		
		
		Tripulant tripulant = daoTripulant.cercarTripulant(login);

        return tripulant;		
	}	


    /**
     * Mtode que serveix per a cercar un vaixell a partir del seu identificador.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @return Vaixell retorna un vaixell.
     * @throws ExcepcioInformacio
     */		
	public Vaixell cercarVaixell(Integer idVaixell) throws ExcepcioInformacio {

		LOGGER.info("cercarVaixell");	
		
        //Cercar un vaixell per el seu identificador (clau primaria de la taula Vaixell). 
		Vaixell vaixell = daoVaixell.cercarVaixell(idVaixell);
        
		return vaixell;		
	}

	
    /**
     * Mtode que serveix per a cercar una rea a partir del seu identificador.
     *
     * @param idArea objecte que cont el identificador de l'rea.
     * @return Area retorna una area.
     * @throws ExcepcioInformacio
     */		
	public Area cercarArea(Integer idArea) throws ExcepcioInformacio {

		LOGGER.info("cercarArea");
		
		Area area = new Area();
 
		if(idArea != -1) {
			
			area = daoArea.cercarArea(idArea);
			
		} else {
			area.setIdArea(-1);
			area.setCodiArea("No assignada");
		}
		
		return area;		
	}
	
	
    /**
     * Mtode que serveix per a cercar un moll a partir del seu identificador.
     *
     * @param idMoll objecte que cont el identificador del moll.
     * @return Moll retorna una moll.
     * @throws ExcepcioInformacio
     */		
	public Moll cercarMoll(Integer idMoll) throws ExcepcioInformacio {

		LOGGER.info("cercarMoll");	

		Moll moll = new Moll();
		
		if(idMoll != -1) {		
		
			moll = daoMoll.cercarMoll(idMoll);
			
		} else {
			moll.setIdMoll(-1);
			moll.setCodiMoll("No assignat");
			moll.setNomPort("No assignat");
		}
        
		return moll;		
	}
		
		
}
