package edu.uoc.grp.service;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.util.LinkedHashMap;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import edu.uoc.grp.exceptions.ExcepcioPlanificacio;
import edu.uoc.grp.dao.DaoTripulant;
import edu.uoc.grp.dao.DaoJornada;
import edu.uoc.grp.dao.DaoVaixell;
import edu.uoc.grp.dao.DaoArea;
import edu.uoc.grp.dao.DaoMoll;
import edu.uoc.grp.model.Tripulant;
import edu.uoc.grp.model.Vaixell;
import edu.uoc.grp.model.Area;
import edu.uoc.grp.model.Moll;
import edu.uoc.grp.utilities.Utils;


/**
 * Classe ServeiPlanificacio.
 * 
 * Classe de tipus servei que proporciona els mtodes necessaris per a gestionar la planificaci de les jornades pesqueres. 
 *
 * @version 1.0; java version "1.8.0";
 */
@Service("ServeiPlanificacio")
public class ServeiPlanificacio {

	private static final Logger LOGGER = LogManager.getLogger(ServeiPlanificacio.class.getName());
	private static final String ALTA = "ALTA";
	private static final String MODIFICAR = "MODIFICAR";
	private static final String PATRO = "Patro";
	private static final String MARINER = "Mariner";	

	@Autowired	
	private DaoTripulant daoTripulant;	
	
	@Autowired	
	private DaoJornada daoJornada;

	@Autowired
	private DaoVaixell daoVaixell;
	
	@Autowired	
	private DaoArea daoArea;

	@Autowired	
	private DaoMoll daoMoll;

    @Autowired
    private Utils utils;
	

    /**
     * Mtode que serveix per a obtenir un llistat d'objectes de tipus vaixell.
     *
     * @return List<Vaixell> retorna un llistat d'objectes de tipus vaixell.
     * @throws ExcepcioPlanificacio
     */	
	public List<Vaixell> obtenirLlistatVaixells() throws ExcepcioPlanificacio {
		
		LOGGER.info("obtenirLlistatVaixells");		
		
        List<Vaixell> llistatVaixell = new ArrayList<Vaixell>();
		llistatVaixell = daoVaixell.obtenirLlistatVaixells();
		
		return llistatVaixell;
	}


    /**
     * Mtode que serveix per a cercar un vaixell a partir del seu nom.
     *
     * @param nomVaixell objecte que cont el nom del vaixell.
     * @return Vaixell retorna un vaixell.
     * @throws ExcepcioPlanificacio
     */		
	public Vaixell cercarVaixell(String nomVaixell) throws ExcepcioPlanificacio {

		LOGGER.info("cercarVaixell");
		
		if(nomVaixell != null) nomVaixell = nomVaixell.trim();
		
		Boolean existeixVaixell = daoVaixell.vaixellExisteix(nomVaixell);
		if(!existeixVaixell) return null;
		
        //Cercar un vaixell per el seu nom (camp amb valor nic de la taula Vaixell). 
		Vaixell vaixell = daoVaixell.cercarVaixell(nomVaixell);
        
		return vaixell;		
	}


    /**
     * Mtode que serveix per obtenir les rees que estiguin disponibles.
     *
     * @return Map<String, String> retorna un mapa amb les rees disponibles.
     * @throws ExcepcioPlanificacio
     */		
	public Map<String, String> obtenirAreesDisponibles() throws ExcepcioPlanificacio {
		
		LOGGER.info("obtenirAreesDisponibles");

		Map<String, String> areesDisponiblesMap = new LinkedHashMap<String, String>();		
        List<Area> llistatAreesDisponibles = daoArea.obtenirAreesDisponibles();
		
        for (Area area : llistatAreesDisponibles) {
    		areesDisponiblesMap.put(area.getIdArea() + "", area.getCodiArea());
        }		
		
		return areesDisponiblesMap;
	}	


    /**
     * Mtode que serveix per assignar una rea a un vaixell.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @param idArea objecte que cont el identificador del rea.
     * @throws ExcepcioPlanificacio
     */			
	public void assignarAreaVaixell(Integer idVaixell, Integer idArea) throws ExcepcioPlanificacio {

		LOGGER.info("assignarAreaVaixell");
		LOGGER.debug("idVaixell: " + idVaixell);	
		LOGGER.debug("idArea: " + idArea);	
		
		try {
		
			Boolean existeixJornada = daoJornada.jornadaExisteix(idVaixell);
			
			if(existeixJornada) {
				
				//dessasignar l'rea que tingui assignada el vaixell.
				if(idArea == -1) daoJornada.modificaJornada("AREA", idVaixell, null, null);
	
				//assignar al vaixell l'rea escollida per l'usuari.
				else daoJornada.modificaJornada("AREA", idVaixell, null, idArea);
				
			} else {
				daoJornada.altaJornada(idVaixell, null, idArea);			
			} 


        } catch (ExcepcioPlanificacio ep) {

			if (ep.getMessage().toUpperCase().indexOf("DUPLICATE ENTRY") != -1) {
				
	        	String missatgeError = utils.getMsg("msg.error.concurrencia.assig.area");
	        	LOGGER.error(missatgeError);
				throw new ExcepcioPlanificacio(missatgeError);
				
			} else {
				
	        	String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();
	        	LOGGER.error(missatgeError);
				throw new ExcepcioPlanificacio(missatgeError);
			}
        }		
	}	


    /**
     * Mtode que serveix per obtenir els molls que estiguin disponibles.
     *
     * @return Map<String, String> retorna un mapa amb els molls disponibles.
     * @throws ExcepcioPlanificacio
     */		
	public Map<String, String> obtenirMollsDisponibles() throws ExcepcioPlanificacio {
		
		LOGGER.info("obtenirMollsDisponibles");

		Map<String, String> mollsDisponiblesMap = new LinkedHashMap<String, String>();		
        List<Moll> llistatMollsDisponibles = daoMoll.obtenirMollsDisponibles();
		
        for (Moll moll : llistatMollsDisponibles) {
        	mollsDisponiblesMap.put(moll.getIdMoll() + "", moll.getCodiMoll());
        }		
		
		return mollsDisponiblesMap;
	}	


    /**
     * Mtode que serveix per assignar un moll a un vaixell.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @param idMoll objecte que cont el identificador del moll.
     * @throws ExcepcioPlanificacio
     */			
	public void assignarMollVaixell(Integer idVaixell, Integer idMoll) throws ExcepcioPlanificacio {

		LOGGER.info("assignarMollVaixell");
		LOGGER.debug("idVaixell: " + idVaixell);
		LOGGER.debug("idMoll: " + idMoll);
		
		try {
		
			Boolean existeixJornada = daoJornada.jornadaExisteix(idVaixell);
			
			if(existeixJornada) {
				
				//dessasignar el moll que tingui assignat el vaixell.
				if(idMoll == -1) daoJornada.modificaJornada("MOLL", idVaixell, null, null);
	
				//assignar al vaixell el moll escollit per l'usuari.
				else daoJornada.modificaJornada("MOLL", idVaixell, idMoll, null);
				
			} else {
				daoJornada.altaJornada(idVaixell, idMoll, null);			
			} 
		
		
        } catch (ExcepcioPlanificacio ep) {

			if (ep.getMessage().toUpperCase().indexOf("DUPLICATE ENTRY") != -1) {
				
	        	String missatgeError = utils.getMsg("msg.error.concurrencia.assig.moll");
	        	LOGGER.error(missatgeError);
				throw new ExcepcioPlanificacio(missatgeError);
				
			} else {
				
	        	String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();
	        	LOGGER.error(missatgeError);
				throw new ExcepcioPlanificacio(missatgeError);
			}
        }
	
	}


    /**
     * Mtode que serveix per comprovar si un vaixell t una tripulaci assignada.
     * Si el vaixell t una tripulaci assignada l'operaci s modificar.
     * Si el vaixell no t una tripulaci assignada l'operaci s alta.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @return String retorna un objecte que cont l'operaci (alta o modificar).
     * @throws ExcepcioPlanificacio 
     */			
	public String comprovarTripulacioAssigVaixell(Integer idVaixell) throws ExcepcioPlanificacio {
		
		String operacio = ALTA;
			
		Boolean tripAssigVaixell = daoTripulant.tripulacioAssigVaixell(idVaixell);
		
		if(tripAssigVaixell) operacio = MODIFICAR;
		
		return operacio;
	}


    /**
     * Mtode que serveix per obtenir els patrons disponibles, i tamb el patr assignat a un vaixell en concret.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @return ArrayList<Object> retorna un llistat d'objectes que contenen patrons.
     * @throws ExcepcioPlanificacio
     */			
	public ArrayList<Object> obtenirPatrons(Integer idVaixell, String operacio) throws ExcepcioPlanificacio {
		
		LOGGER.info("obtenirPatrons");
		
		ArrayList<Object> resultat = new ArrayList<Object>();
        Map<String, String> patronsMap = new LinkedHashMap<String, String>();
		patronsMap.put("-1", "- No Assignat -");
        String patroAssignat = "";
		
        List<Tripulant> llistatPatrons = new ArrayList<Tripulant>(); 
        
        if (ALTA.equalsIgnoreCase(operacio)) {   
		
        	llistatPatrons = daoTripulant.obtenirTripDisponiblesAlta(idVaixell, PATRO);
		
        } else if (MODIFICAR.equalsIgnoreCase(operacio)) {
        	
        	llistatPatrons = daoTripulant.obtenirTripDisponiblesModificar(idVaixell, PATRO);
        	
        }
		
        for (Tripulant tripulant : llistatPatrons) {
        	
    		LOGGER.debug("tripulant - id: " + tripulant.getIdTripulant() + " - NomComplet: " + tripulant.getNomComplet() + " - Tipus: " + tripulant.getTipusTripulant() + " VaixellAssignat: " + tripulant.getVaixellAssignat());
    		
            if (tripulant.getVaixellAssignat() != null) {
            	
            	patroAssignat = tripulant.getIdTripulant() + "";

            }
            
            patronsMap.put(tripulant.getIdTripulant() + "", tripulant.getNomComplet());            
        }

        resultat.add(0, patronsMap);
        resultat.add(1, patroAssignat);

        return resultat;		
	}		


    /**
     * Mtode que serveix per obtenir els mariners disponibles, i tamb els mariners assignats a un vaixell en concret.
     *
     * @param idVaixell objecte que cont el identificador del vaixell.
     * @return ArrayList<Map<String, String>> retorna un llistat d'objectes que contenen mariners.
     * @throws ExcepcioPlanificacio
     */			
	public ArrayList<Map<String, String>> obtenirMariners(Integer idVaixell, String operacio) throws ExcepcioPlanificacio {
		
		LOGGER.info("obtenirMariners");
		
		ArrayList<Map<String, String>> resultat = new ArrayList<Map<String, String>>();
        Map<String, String> marinersMap = new LinkedHashMap<String, String>();
        Map<String, String> marinersAssignatsMap = new LinkedHashMap<String, String>();
        marinersMap.put("-1", "- No Assignat -");        
		
        List<Tripulant> llistatMariners = new ArrayList<Tripulant>();
        
        if (ALTA.equalsIgnoreCase(operacio)) {
        	
        	llistatMariners = daoTripulant.obtenirTripDisponiblesAlta(idVaixell, MARINER);
        	
        } else if (MODIFICAR.equalsIgnoreCase(operacio)) {
        	
        	llistatMariners = daoTripulant.obtenirTripDisponiblesModificar(idVaixell, MARINER);        	
        }

        int i = 1;
        for (Tripulant tripulant : llistatMariners) {
        	
    		LOGGER.debug("tripulant - id: " + tripulant.getIdTripulant() + " - NomComplet: " + tripulant.getNomComplet() + " - Tipus: " + tripulant.getTipusTripulant());
    		
            if (tripulant.getVaixellAssignat() != null) {
            	marinersAssignatsMap.put("MARINER_" + i, tripulant.getIdTripulant() + "");
                i++;
            }
            
            marinersMap.put(tripulant.getIdTripulant() + "", tripulant.getNomComplet());            
        }

        resultat.add(0, marinersMap);
        resultat.add(1, marinersAssignatsMap);

        return resultat;		
	}		


    /**
     * Mtode que serveix per a assignar una tripulaci a un vaixell en concret.
     *
     * @param idVaixell  objecte que cont el indentificador del vaixell.
     * @param patro      objecte que cont el indentificador del patr.
     * @param mariner1   objecte que cont el indentificador del mariner1.
     * @param mariner2   objecte que cont el indentificador del mariner2.
     * @param mariner3   objecte que cont el indentificador del mariner3.
     * @param mariner4   objecte que cont el indentificador del mariner4.
     * @param mariner5   objecte que cont el indentificador del mariner5.
     * @throws ExcepcioPlanificacio
     */	
    @Transactional(rollbackFor={ExcepcioPlanificacio.class})	
	public void assignarTripulacioVaixell(Integer idVaixell, Integer patro, Integer mariner1, Integer mariner2, Integer mariner3, Integer mariner4, Integer mariner5) throws ExcepcioPlanificacio {

		LOGGER.info("assignarTripulacioVaixell");
		
        List<Integer> tripulacio = new ArrayList<Integer>();
        tripulacio.add(patro);     tripulacio.add(mariner1);        
        tripulacio.add(mariner2);  tripulacio.add(mariner3);
        tripulacio.add(mariner4);  tripulacio.add(mariner5);
        	
		try {
			
			daoTripulant.actualitzaTripulant(idVaixell, tripulacio);
        
        } catch (ExcepcioPlanificacio ep) {
        	
	    	if (ep.getMessage().toUpperCase().indexOf("CONCURRENCY_ERROR") != -1) {
	    		
	    		String missatgeError = utils.getMsg("msg.error.concurrencia.assig.trip");
	    		LOGGER.error(missatgeError);	    		
	    		throw new ExcepcioPlanificacio(missatgeError);
	    		
	    	} else {
	    		
	    		String missatgeError = utils.getMsg("msg.error.general.bd") + ep.getMessage();
	    		LOGGER.error(missatgeError);	    		
	    		throw new ExcepcioPlanificacio(missatgeError);	
	    	}        
        }        
    }


    /**
     * Mtode que serveix per a desfer l'assignaci d'una tripulaci a un vaixell en concret.
     *
     * @param idVaixell objecte que cont l'indentificador del vaixell.
     * @throws ExcepcioPlanificacio
     */	
	public void baixaTripulacio(Integer idVaixell) throws ExcepcioPlanificacio {

		LOGGER.info("baixaTripulacio");
		LOGGER.debug("idVaixell: " + idVaixell);		

        daoTripulant.baixaTripulacio(idVaixell);
	}	


}
