package edu.uoc.grp.service;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;
import java.util.Map;
import edu.uoc.grp.exceptions.ExcepcioUsuari;
import edu.uoc.grp.dao.DaoUsuari;
import edu.uoc.grp.model.Usuari;
import edu.uoc.grp.model.Empleat;
import edu.uoc.grp.utilities.Utils;


/**
 * Classe ServeiUsuari.
 * 
 * Classe de tipus servei que proporciona els mtodes necessaris per a gestionar el usuari de l'aplicaci. 
 *
 * @version 1.0; java version "1.8.0";
 */
@Service("ServeiUsuari")
public class ServeiUsuari {
	
	private static final Logger LOGGER = LogManager.getLogger(ServeiUsuari.class.getName());	
	
	@Autowired
	private DaoUsuari daoUsuari;

    @Autowired
    private Utils utils;
	

    /**
     * Mtode que serveix per a obtenir un llistat d'objectes de tipus usuari.
     *
     * @return List<Usuari> retorna un llistat d'objectes de tipus usuari.
     * @throws ExcepcioUsuari
     */	
	public List<Usuari> obtenirLlistatUsuaris() throws ExcepcioUsuari {
		
		LOGGER.info("obtenirLlistatUsuaris");		
		
        List<Usuari> llistatUsuari = new ArrayList<Usuari>();
		llistatUsuari = daoUsuari.obtenirLlistatUsuaris();
		
		return llistatUsuari;
	}


    /**
     * Mtode que serveix per a obtenir un llistat d'objectes de tipus empleat.
     *
     * @return List<Empleat> retorna un llistat d'objectes de tipus empleat.
     * @throws ExcepcioUsuari
     */	
	public List<Empleat> obtenirLlistatEmpleats() throws ExcepcioUsuari {
		
		LOGGER.info("obtenirLlistatEmpleats");		
		
        List<Empleat> llistatEmpleat = new ArrayList<Empleat>();
        llistatEmpleat = daoUsuari.obtenirLlistatEmpleats();
		
		return llistatEmpleat;
	}	


    /**
     * Mtode que serveix per modificar el estat (actiu o baixa) de un usuari en concret.
     *
     * @param usuariAdministrador objecte que cont el nom identificatiu del usuari administrador.
     * @param login objecte que cont el nom identificatiu del usuari.
     * @param estatUsuari objecte que cont el estat del usuari.
     * @throws ExcepcioUsuari
     */			
	public void actualitzaEstatUsuari(String usuariAdministrador, String login, String estatUsuari) throws ExcepcioUsuari {
		
		LOGGER.info("actualitzaEstatUsuari");		
        
		daoUsuari.actualitzaEstatUsuari(usuariAdministrador, login, estatUsuari);
	}


    /**
     * Mtode que serveix per modificar les dades de un usuari en concret.
     *
     * @param usuariAdministrador objecte que cont el nom identificatiu del usuari administrador.
     * @param idUsuari objecte que cont el identificador del usuari.
     * @param login objecte que cont el nom identificatiu del usuari.
     * @param contrasenya objecte que cont la contrasenya del usuari.
     * @param eMail objecte que cont el correu electrnic del usuari. 
     * @return Map<String, String> retorna un mapa que cont el resultat de la modificaci d'usuari.
     */			
	public Map<String, String> modificaUsuari(String usuariAdministrador, Integer idUsuari, String login, String contrasenya, String eMail) {
		
		LOGGER.info("modificaUsuari");
		
		Map<String, String> resultat = new HashMap<String, String>();
        
		try {
		
			daoUsuari.modificaUsuari(usuariAdministrador, idUsuari, login, contrasenya, eMail);
        
			resultat.put("RESULTAT", "OK");
        
			resultat.put("MISSATGE", utils.getMsg("msg.info.mod.usuari.ok"));
	
		} catch(ExcepcioUsuari eu) {
			
			if (eu.getMessage().toUpperCase().indexOf("DUPLICATE ENTRY") != -1) {
	
	            resultat.put("RESULTAT", "NOK");
	            resultat.put("MISSATGE", utils.getMsg("msg.error.id.usuari.duplicat"));
				
	        } else {
	        	
	            resultat.put("RESULTAT", "NOK");
	            resultat.put("MISSATGE", utils.getMsg("msg.error.general.bd") + eu.getMessage());
	        }
	
		}
	
		return resultat;			
	}


    /**
     * Mtode que serveix per donar de alta a un nou usuari de l'aplicaci.
     *
     * @param usuariAdministrador objecte que cont el nom identificatiu del usuari administrador.
     * @param idUsuari objecte que cont el identificador del usuari.
     * @param tipusUsuari objecte que cont el tipus de usuari. 
     * @param login objecte que cont el nom identificatiu del usuari.
     * @param contrasenya objecte que cont la contrasenya del usuari.
     * @param eMail objecte que cont el correu electrnic del usuari. 
     * @return Map<String, String> retorna un mapa que cont el resultat de l'alta d'usuari.
     */			
	public Map<String, String> altaUsuari(String usuariAdministrador, Integer idUsuari, String tipusUsuari, String login, String contrasenya, String eMail) {
		
		LOGGER.info("altaUsuari");		
        
		Map<String, String> resultat = new HashMap<String, String>();
		
		try {
		
			daoUsuari.altaUsuari(usuariAdministrador, idUsuari, tipusUsuari, login, contrasenya, eMail);
			
	        resultat.put("RESULTAT", "OK");
	        resultat.put("MISSATGE", utils.getMsg("msg.info.alta.usuari.ok"));
		
		} catch(ExcepcioUsuari eu) {
			
			if (eu.getMessage().toUpperCase().indexOf("DUPLICATE ENTRY") != -1) {

                resultat.put("RESULTAT", "NOK");
                resultat.put("MISSATGE", utils.getMsg("msg.error.id.usuari.duplicat"));
				
            } else {
            	
                resultat.put("RESULTAT", "NOK");
                resultat.put("MISSATGE", utils.getMsg("msg.error.general.bd") + eu.getMessage());
            }

		}
		
		return resultat;		
	}


    /**
     * Mtode que serveix per cercar a un usuari en concret utilitzant el seu nom identificatiu d'usuari.
     *
     * @param login objecte que cont el nom identificatiu del usuari.
     * @return Map<String, Object> retorna un mapa que cont els resultats de la cerca.
     */		
	public Map<String, Object> cercarUsuari(String login) {

		LOGGER.info("cercarUsuari");		
		LOGGER.debug("cercarUsuari: login: " + login);
		
		if(login != null) login = login.trim();
		
		Map<String, Object> resultat = new HashMap<String, Object>();
        String missatge = " ";
        resultat.put("RESULTAT", " ");
        resultat.put("MISSATGE", missatge);
        Empleat empleat = null;        
        Usuari usuari = null;
        Boolean existeixUsuari = false;
        
        try {
        	
        	existeixUsuari = daoUsuari.usuariExisteix(login);
        	
        } catch (ExcepcioUsuari eu) {
            missatge = utils.getMsg("msg.error.general.bd") + eu.getMessage();
            resultat.put("RESULTAT", "NOK");
            resultat.put("MISSATGE", missatge);
            return resultat;
        }
        
        //L'usuari no existeix dins de la base de dades
        if (!existeixUsuari) {
            missatge = utils.getMsg("msg.error.usuari.no.trobat.bd");
            resultat.put("RESULTAT", "NOK");
            resultat.put("MISSATGE", missatge);

        //L'usuari existeix dins de la base de dades
        } else {

            try {
            	//Cercar les dades del usuari, incloent les seves dades personals (les dades de empleat). 
            	usuari = daoUsuari.cercarUsuari(login);
    			empleat = daoUsuari.cercarEmpleat(usuari.getIdUsuari());
    			
                resultat.put("RESULTAT", "OK");
                resultat.put("USUARI", usuari);
                resultat.put("EMPLEAT", empleat);

            } catch (ExcepcioUsuari eu) {
                missatge = utils.getMsg("msg.error.general.bd") + eu.getMessage();
                resultat.put("RESULTAT", "NOK");
                resultat.put("MISSATGE", missatge);
                return resultat;
                
            } catch (Exception e) {
                missatge = utils.getMsg("msg.error.general") + e.getMessage();
                resultat.put("RESULTAT", "NOK");
                resultat.put("MISSATGE", missatge);
                return resultat;
            }
        
        }
        
		return resultat;
	}


    /**
     * Mtode que serveix per a cercar un empleat a partir del seu identificador de empleat.
     *
     * @param idEmpleat objecte que cont el identificador de empleat.
     * @return Empleat retorna un empleat.
     * @throws ExcepcioUsuari
     */		
	public Empleat cercarEmpleat(Integer idEmpleat) throws ExcepcioUsuari {

		LOGGER.info("cercarEmpleat");		
		
        //Cercar el empleat per el seu idEmpleat (clau primaria de la taula Empleat). 
         Empleat empleat = daoUsuari.cercarEmpleat(idEmpleat);
        
		return empleat;		
	}


    /**
     * Mtode que serveix per comprovar si la contrasenya proporcionada s idntica a l'emmagatzemada dins de la base de dades.
     *
     * @param login objecte que cont el nom identificatiu del usuari.
     * @param contrasenya objecte que cont la contrasenya del usuari.
     * @return Boolean retorna cert si la contrasenya s correcta, altrament retorna fals.
     * @throws ExcepcioUsuari
     */		
	public Boolean comprovarContrasenya(String login, String contrasenya) throws ExcepcioUsuari {

		LOGGER.info("comprovarContrasenya");		
		Boolean contrasenyaValida = false;
 
		Usuari usuari = daoUsuari.cercarUsuari(login);
        
		if (contrasenya.equals(usuari.getContrasenya())) contrasenyaValida = true;
		
		return contrasenyaValida;		
	}		


    /**
     * Mtode que serveix per canviar la contrasenya de un usuari en concret a partir del seu nom identificatiu de usuari.
     *
     * @param login objecte que cont el nom identificatiu del usuari.
     * @param contrasenya objecte que cont la contrasenya del usuari.
     * @throws ExcepcioUsuari
     */		
	public void canviarContrasenya(String login, String contrasenya) throws ExcepcioUsuari {

		LOGGER.info("canviarContrasenya");		

		daoUsuari.canviarContrasenya(login, login, contrasenya);
	}		

	
}
